# coding: utf-8
from django.db import transaction
from django.db.models.base import ModelBase


def model_modifier_metaclass(meta_base=ModelBase, **params):
    u"""Возвращает метакласс, изменяющий параметры полей модели.

    :param dict params: Словарь с новыми значениями параметров полей. Ключ
        словаря должен содержать имя поля в модели (*field.attname*), а
        значение - словарь с новыми параметрами поля.

    .. note::

       Пример использования:

       .. code::

          class BaseModel(models.Model):
              name = models.CharField(max_length=150)

          modified_model_params = {
              'name': {
                  'max_length': 300
              }
          }
          class MyModel(BaseModel):
              # Модель с увеличенной до 300 символов длиной поля name.
              __metaclass__ = model_modifier_metaclass(**modified_model_params)

              class Meta:
                  verbose_name = u'Образец справочника'
    """
    class ModifiedModelBase(meta_base):
        def __new__(cls, name, bases, attrs):
            model = super(ModifiedModelBase, cls).__new__(
                cls, name, bases, attrs
            )

            for field_name, field_params in params.iteritems():
                field = model._meta.get_field(field_name)
                for param_name, param_value in field_params.iteritems():
                    assert hasattr(field, param_name), param_name
                    setattr(field, param_name, param_value)

            return model

    return ModifiedModelBase


def nested_commit_on_success(func):
    u"""Аналог commit_on_success, не завершающий существующую транзакцию.

    Этот декоратор используется для выполнения функции внутри транзакции. В
    случае, когда функция выполнена без ошибок, транзакция завершается, иначе
    выполняется откат транзакции.

    В отличие от стандартного декоратора commit_on_success, эта версия сначала
    проверяет, не была ли ранее начата другая транзакция. Если да, то по
    завершении функции не будет происходить завершения/отката транзакции.

    https://djangosnippets.org/snippets/1343/
    """
    def _nested_commit_on_success(*args, **kwargs):
        if transaction.is_managed():
            return func(*args, **kwargs)
        else:
            commit_on_success = transaction.commit_on_success(func)
            return commit_on_success(*args, **kwargs)
    return transaction.wraps(func)(_nested_commit_on_success)
